package de.ozgcloud.eingang.enterprise.entry;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.ArgumentMatchers.*;
import static org.mockito.Mockito.*;

import java.io.InputStream;

import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.Mockito;
import org.mockito.Spy;

import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.datatype.jsr310.JavaTimeModule;

import de.ozgcloud.common.test.TestUtils;
import de.ozgcloud.eingang.common.formdata.FormData;
import de.ozgcloud.eingang.common.formdata.FormDataTestFactory;
import lombok.SneakyThrows;

class EntryDataMapperTest {

	@Spy
	@InjectMocks
	private EntryDataMapper mapper;
	@Mock
	private FormDataMapper formDataMapper;

	@Spy
	private ObjectMapper objectMapper = new ObjectMapper()
			.configure(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES, true)
			.registerModule(new JavaTimeModule());

	@Nested
	class TestMappingEntryData {

		private InputStream jsonInput = TestUtils.loadFile("request/simple.json");

		@Nested
		class TestReadRequest {
			@Test
			void shouldReadJson() {
				var read = mapper.readRequest(jsonInput);

				assertThat(read).usingRecursiveComparison()
						.isEqualTo(EntryDataTestFactory.createBuilder().control(ControlDataTestFactory.createBuilder().representations(null).build())
								.build());
			}
		}

		@Test
		void shouldReadRequest() {
			mapper.mapEntryData(jsonInput);

			verify(mapper).readRequest(jsonInput);
		}

		@Test
		@SneakyThrows
		void shouldCallFormDataMapper() {
			var entryData = EntryDataTestFactory.create();
			doReturn(entryData).when(objectMapper).readValue(any(InputStream.class), Mockito.<Class<EntryData>>any());

			mapper.mapEntryData(jsonInput);

			verify(formDataMapper).mapEntryData(entryData);
		}

		@Test
		void shouldReturnMappedResult() {
			FormData formData = FormDataTestFactory.create();
			when(formDataMapper.mapEntryData(any())).thenReturn(formData);

			var result = mapper.mapEntryData(jsonInput);

			assertThat(result).isSameAs(formData);
		}
	}

}
