/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.eingang.semantik.enginebased.afm;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import de.ozgcloud.eingang.semantik.common.ServiceKontoFactory;

public class AfmHeaderTestFactory {

	public static final String ID = UUID.randomUUID().toString();
	public static final String TIMESTAMP = "2020-11-18T09:09:27.627Z";
	public static final String FORM_ID = "waffen/kleinerWaffenschein";
	public static final String FORM = "Kleiner Waffenschein gem. § 10 Abs. 4 Satz 4 Waffengesetz (WaffG)";
	public static final String SENDER = "afm.schleswig-holstein.de";

	public static final String CUSTOM_POSTFACH_ID = "postfach_id";
	public static final String CUSTOM_VORNAME = "vorname";
	public static final String CUSTOM_NACHNAME = "nachname";
	public static final String CUSTOM_GEBURTSNAME = "Geburtsname";
	public static final String CUSTOM_GEBURTSORT = "geburtsort";
	public static final String CUSTOM_EMAIL = "email";
	public static final String CUSTOM_TELEFON = "telefon";
	public static final String CUSTOM_STRASSE = "strasse";
	public static final String CUSTOM_PLZ = "plz";
	public static final String CUSTOM_ORT = "ort";

	public static final String POSTFACH_NAME_ID = "name-id-value";

	public static final int REST_RESPONSE_NAME_MEMBER_SCOPE_MAILBOX_TYPE_VALUE = 1;

	public static Map<String, Object> createFormDataMap() {
		var map = new HashMap<String, Object>();
		map.put(AfmHeaderMapper.HEADER_FIELD, createHeaderMap());
		map.put(AfmHeaderMapper.POSTFACH_NAME_ID, POSTFACH_NAME_ID);
		map.put(ServiceKontoFactory.REST_RESPONSE_NAME, List.of(createRestResponseNameMap()));

		return map;
	}

	@SuppressWarnings("unchecked")
	public static Map<String, Object> createFormDataMapWithExtendedHeaders() {
		var map = new HashMap<>(createFormDataMap());
		((Map<String, Object>) map.get(AfmHeaderMapper.HEADER_FIELD)).putAll(createCustomHeaderMap());
		return map;
	}

	public static Map<String, Object> createHeaderMap() {
		var map = new HashMap<String, Object>();
		map.put(AfmHeaderMapper.ID, ID);
		map.put(AfmHeaderMapper.TIMESTAMP, TIMESTAMP);
		map.put(AfmHeaderMapper.FORM_ID, FORM_ID);
		map.put(AfmHeaderMapper.FORM, FORM);
		map.put(AfmHeaderMapper.SENDER, SENDER);
		return map;
	}

	public static Map<String, Object> createCustomHeaderMap() {
		var map = new HashMap<String, Object>();
		map.put(ServiceKontoFactory.KEY_BAYERN_ID_POSTFACH_ID, CUSTOM_POSTFACH_ID);
		map.put(AfmAntragstellerHeaderMapper.KEY_VORNAME, CUSTOM_VORNAME);
		map.put(AfmAntragstellerHeaderMapper.KEY_NACHNAME, CUSTOM_NACHNAME);
		map.put(AfmAntragstellerHeaderMapper.KEY_GEBURTSNAME, CUSTOM_GEBURTSNAME);
		map.put(AfmAntragstellerHeaderMapper.KEY_GEBURTSORT, CUSTOM_GEBURTSORT);
		map.put(AfmAntragstellerHeaderMapper.KEY_EMAIL, CUSTOM_EMAIL);
		map.put(AfmAntragstellerHeaderMapper.KEY_TELEFON, CUSTOM_TELEFON);
		map.put(AfmAntragstellerHeaderMapper.KEY_STRASSE, CUSTOM_STRASSE);
		map.put(AfmAntragstellerHeaderMapper.KEY_PLZ, CUSTOM_PLZ);
		map.put(AfmAntragstellerHeaderMapper.KEY_ORT, CUSTOM_ORT);
		return map;
	}

	public static Map<String, Object> createRestResponseNameMap() {
		return Map.of(ServiceKontoFactory.REST_RESPONSE_NAME_MEMBER_SCOPE, List.of(createRestResponseNameMemberScopeMap()));
	}

	private static Map<String, Object> createRestResponseNameMemberScopeMap() {
		return Map.of(ServiceKontoFactory.REST_RESPONSE_NAME_MEMBER_SCOPE_MAILBOX_TYPE, REST_RESPONSE_NAME_MEMBER_SCOPE_MAILBOX_TYPE_VALUE);
	}
}